define(
  'structure/testy/TestStatusFilter',
  [
    'jquery',
    'underscore',
    'almworks/util/Util'
  ],
  function (AJS$, _, U) {
    'use strict';
    function TestStatusFilter(runOptions, statusOptions) {
      var TestyUtils = window.namespace('almworks.testy.utils');
      var structure =  TestyUtils.structure();
      var $ = structure.$;

      function findStructure() {
        return structure.instances[0].pageManager.getFocusedPanel().getStructureId();
      }

      function fit(items) { // !!items === true
        return _.map(items, function (item) {
          item.text = item.name;
          return item;
        });
      }

      function checkError(result, message) {
        if (!result || !!result && result.error || !_.isArray(result)) {
          !!window.console && window.console.error(message);
          return true;
        }
        return false;
      }

      function createTestRunSelect(options, structureId) {
        TestyUtils.rest('GET', 'run/search?structure=' + structureId, null, function (testRuns) {
          if (checkError(testRuns, "Failed to load test runs for structure " + structureId + ": "  + testRuns)) {
            return;
          }

          var input$ = AJS$('#' + options.inputId);
          if (_.isEmpty(testRuns)) {
            input$.attr('disabled', 'disabled');
            $('#' + options.fieldContainerId).addClass('disabled');
            input$.attr('placeholder', options.noTestRunsFound)
          } else {
            input$.val(options.selectedId || testRuns[0].id || '');
          }

          input$.auiSelect2({
            data: fit(testRuns)
          });
        });
      }

      /* options.selectedIds is comma separated values */
      function createStatusSelect(options) {
        TestyUtils.rest('GET', 'status', null, function (testStatuses) {
          if (checkError(testStatuses, "Failed to load Testy statuses: " + testStatuses)) {
            return;
          }

          var input$ = AJS$('#' + options.inputId);
          input$.auiSelect2({
            multiple: true,
            data: _.sortBy(fit(testStatuses), 'selectorOrder'),
            containerCssClass: 'tty-status-selection',
            formatSelectionCssClass: function (data, div$) { // LI class
              div$.addClass('tty-' + data.colorSet);
              div$.addClass('tty-status-el');
            },
            dropdownCssClass: 'tty-status-dropdown',
            formatResult: function (status, container, query, escapeMarkup) {
                return '<div class="tty-status-el tty-' + status.colorSet + '">'
                    + escapeMarkup(status.name) + '</div>';
            }
          });
          input$.val(options.selectedIds).trigger('change');
        });
      }

      return U.extend(this, {
        install: function () {
          var structureId = findStructure();
          createTestRunSelect(runOptions, structureId);
          createStatusSelect(statusOptions);
        }
      });
    }
    return TestStatusFilter;
});
